# (c) 2018-2024 MPIB <https://www.mpib-berlin.mpg.de/>,
#     2018-2019 MPI-CBS <https://www.cbs.mpg.de/>,
#     2018-2019 MPIP <http://www.psych.mpg.de/>
#
# This file is part of Castellum.
#
# Castellum is free software; you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Castellum is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Castellum. If not, see
# <http://www.gnu.org/licenses/>.

import datetime

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import SuspiciousOperation
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import gettext_noop
from django.views.generic import CreateView
from django.views.generic import UpdateView

from castellum.audit.helpers import audit
from castellum.castellum_auth.mixins import PermissionRequiredMixin
from castellum.utils.mail import MailContext

from .forms import AdditionalInfoForm
from .forms import ConsentForm
from .forms import DataProtectionForm
from .models import Consent
from .models import ConsentDocument
from .models import Report
from .models import Subject


class SubjectMixin:
    """Use this on every view that represents a subject.

    Requires ``AccessMixin``.

    -   set ``self.subject``
    -   check privacy level
    """

    def get_subject(self):
        obj = self.get_object()
        if isinstance(obj, Subject):
            return obj
        else:
            return obj.subject

    def check_auth_conditions(self):
        if not super().check_auth_conditions():
            return False
        self.subject = self.get_subject()
        return self.request.user.has_privacy_level(self.subject.privacy_level)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['subject'] = self.subject
        return context


class BaseDataProtectionUpdateView(PermissionRequiredMixin, UpdateView):
    model = Subject
    form_class = DataProtectionForm
    permission_required = 'subjects.change_subject'
    template_name = 'subjects/subject_data_protection_form.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def form_valid(self, form):
        if settings.CASTELLUM_GDPR_NOTIFICATION_TO:
            old = Subject.objects.get(pk=self.object.pk)

            with MailContext('internal') as ctx:
                if self.object.export_requested and not old.export_requested:
                    rel = reverse('subjects:detail', args=[self.object.pk])
                    url = self.request.build_absolute_uri(rel)

                    self.object.to_be_deleted_notified = ctx.send_mail(
                        settings.CASTELLUM_EXPORT_SUBJECT_NOTIFICATION_SUBJECT,
                        (
                            lambda: {'url': url},
                            settings.CASTELLUM_EXPORT_SUBJECT_NOTIFICATION_BODY,
                            settings.CASTELLUM_EXPORT_SUBJECT_NOTIFICATION_BODY_EN,
                        ),
                        settings.CASTELLUM_GDPR_NOTIFICATION_TO,
                    )

        audit(
            gettext_noop('subject updated'),
            self.request.user.username,
            subject_pk=self.object.pk,
        )

        return super().form_valid(form)


class BaseConsentChangeView(PermissionRequiredMixin, CreateView):
    model = Consent
    form_class = ConsentForm
    permission_required = 'subjects.change_subject'
    template_name = 'subjects/subject_consent_form.html'

    def get_object(self):
        # weird because model = Consent, but this is only used in
        # SubjectMixin, not in CreateView
        return get_object_or_404(Subject, pk=self.kwargs['pk'])

    def get_form(self):
        if not ConsentDocument.objects.filter(is_valid=True).exists():
            ConsentDocument.objects.create()
        return super().get_form()

    def notify_to_be_deleted(self):
        if not settings.CASTELLUM_GDPR_NOTIFICATION_TO:
            return
        if self.subject.to_be_deleted_notified:
            return
        with MailContext('internal') as ctx:
            rel = reverse('subjects:detail', args=[self.subject.pk])
            url = self.request.build_absolute_uri(rel)

            return ctx.send_mail(
                settings.CASTELLUM_DELETE_SUBJECT_NOTIFICATION_SUBJECT,
                (
                    lambda: {'url': url},
                    settings.CASTELLUM_DELETE_SUBJECT_NOTIFICATION_BODY,
                    settings.CASTELLUM_DELETE_SUBJECT_NOTIFICATION_BODY_EN,
                ),
                settings.CASTELLUM_GDPR_NOTIFICATION_TO,
            )

    def form_valid(self, form):
        Consent.objects.filter(subject=self.subject).delete()
        form.instance.subject = self.subject
        form.instance.underage_when_given = self.subject.contact.is_underage

        if form.cleaned_data['to_be_deleted']:
            if not self.subject.to_be_deleted:
                self.subject.to_be_deleted = datetime.date.today()
                self.subject.to_be_deleted_notified = self.notify_to_be_deleted()
                self.subject.save()
        else:
            if self.subject.to_be_deleted:
                # If a user explicitly removes the ``to_be_deleted`` they
                # expect that this task is done. If there are still reasons
                # for deletion there should be a new notificaition.
                self.subject.to_be_deleted = None
                self.subject.to_be_deleted_notified = False
                self.subject.save()

        audit(
            gettext_noop('consent changed'),
            self.request.user.username,
            subject_pk=self.subject.pk,
        )
        return super().form_valid(form)


class BaseAdditionalInfoUpdateView(PermissionRequiredMixin, UpdateView):
    model = Subject
    form_class = AdditionalInfoForm
    permission_required = 'subjects.change_subject'
    template_name = 'subjects/subject_additional_info_form.html'

    def form_valid(self, form):
        audit(
            gettext_noop('subject updated'),
            self.request.user.username,
            subject_pk=self.subject.pk,
        )
        return super().form_valid(form)


class BaseReportView(PermissionRequiredMixin, CreateView):
    model = Report
    fields = ['additional_wishes']
    template_name = 'subjects/subject_report.html'

    def form_valid(self, form):
        if self.subject.blocked:
            raise SuspiciousOperation

        form.instance.subject = self.subject
        form.instance.user = self.request.user
        obj = form.save()

        self.subject.blocked = True
        self.subject.save()

        path = reverse('subjects:report-detail', args=[obj.pk])
        url = self.request.build_absolute_uri(path)

        with MailContext('internal') as ctx:
            ctx.send_mail(
                settings.CASTELLUM_REPORT_NOTIFICATION_SUBJECT,
                (
                    lambda: {'url': url},
                    settings.CASTELLUM_REPORT_NOTIFICATION_BODY,
                    settings.CASTELLUM_REPORT_NOTIFICATION_BODY_EN,
                ),
                settings.CASTELLUM_REPORT_NOTIFICATION_TO,
            )

        messages.success(self.request, _('Your report has been received.'))
        return redirect(self.get_success_url())
